#!/usr/bin/env python

import os, math, pygame
from pygame.locals import *
from pygame.compat import geterror

game = None
avatar = None
rooms = None
items = None
treasures = None
inventory = None
allsprites = None
messagebox = None
SCREEN_WIDTH = 800
SCREEN_HEIGHT = 620
ROOM_HEIGHT = 510

def load_image(fullname, colorkey=None):
    try:
        image = pygame.image.load(fullname)
    except pygame.error:
        print ('Cannot load image:', fullname)
        raise SystemExit(str(geterror()))
    if colorkey is -2:
        image = image.convert_alpha()
    else:    
        image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image

def load_frame(i, filespec, mode):
    filename = (filespec % i) + ".png"
    return load_image(filename, mode)

def play_sound(filename):
    pygame.mixer.stop()
    filepath = "audio/" + filename + ".wav"
    sound = pygame.mixer.Sound(filepath)
    channel = sound.play()

def inside_polygon(x,y,poly):
    n = len(poly)
    inside =False
    p1x,p1y = poly[0]
    for i in range(n+1):
        p2x,p2y = poly[i % n]
        if y > min(p1y,p2y):
            if y <= max(p1y,p2y):
                if x <= max(p1x,p2x):
                    if p1y != p2y:
                        xinters = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
                    if p1x == p2x or x <= xinters:
                        inside = not inside
        p1x,p1y = p2x,p2y
    return inside

class Game():
    def __init__(self):
        self.cur_room = None
        self.cur_item = ""
        self.pending = ""
        self.screen = None
        self.font = None
        self.is_drag = False
        self.score = 0
        self.gameover = False
        self.sound_on = False

class Splash():
    def __init__(self):
        filename = "squares/splash_new.png"
        self.image = load_image(filename, -2)

    def draw(self, screen):
        screen.blit(self.image, (200, 150))
        

class MessageBox():
    def __init__(self):
        self.border_color = pygame.Color(0xe0e0e0e0)
        self.border_height = 2
        self.contents = Rect(100, 100, 650, 260)
        self.contents_color = pygame.Color("white")
#        self.font = pygame.font.SysFont("timesnewroman", 20)
        self.font = pygame.font.Font("ttf-bitstream-vera-1.10/VeraSeBd.ttf", 14)
        self.button = Rect(390, 320, 50, 30)
        self.ok_text = self.font.render("OK", True, pygame.Color("black"), self.border_color)
        self.ok_rect = Rect(400, 325, 40, 30)
        self.linesize = self.font.get_linesize()
        self.active = False
        self.lines = []
        self.dirty = False

    def draw(self, screen):
        if not self.dirty:
            return
        self.dirty = False
        screen.fill(self.contents_color, self.contents)
        screen.blit(self.image, (110, 110))
        x = 370
        y = 110 + (200 - len(self.lines) * self.linesize) / 2
        for line in self.lines:
            screen.blit(line, (x, y))
            y += self.linesize
        pygame.draw.rect(screen, self.border_color, self.button, 0)
        screen.blit(self.ok_text, self.ok_rect)

    def display(self, image, text, sound):
        filename = "squares/" + image + ".png"
        self.image = load_image(filename, -2)
        self.rect = self.image.get_rect()
        lines = text.splitlines()
        self.lines = []
        for line in lines:
            self.lines.append(self.font.render(line, True, pygame.Color("black"), pygame.Color("white")))
        self.active = True
        self.dirty = True
        if sound and game.sound_on:
            play_sound(sound)
        
    def hide(self):
        if game.gameover:
            pygame.quit()
        if game.score >= 100:
            messagebox.display("banker",  "   Congratulations!!!   \n      YOU WON!!!       ", "won")
            game.gameover = True
            return
        self.active = False
        game.cur_room.draw(game.screen)

    def mouse_click(self, pos):
        if self.button.collidepoint(pos):
            self.hide()


class Inventory:
    def __init__(self):
        self.border_color = pygame.Color("grey")
        self.border_height = 16
        self.contents = Rect(0, ROOM_HEIGHT + self.border_height/2-1, SCREEN_WIDTH, SCREEN_HEIGHT - ROOM_HEIGHT - self.border_height+1) 
        self.contents_color = pygame.Color("white")
        self.font = game.font
        self.border_text = self.font.render("What you have:", True, pygame.Color("black"), self.border_color)
        self.status = self.font.render("You are in the Bank", True, pygame.Color("black"), self.border_color)
        self.score = self.font.render("Score: 0", True, pygame.Color("black"), self.border_color)
        self.items = []
        self.dirty = True
        
    def draw(self, screen):
        if not self.dirty:
            return
        self.dirty = False
        screen.fill(self.contents_color, self.contents)
        pygame.draw.rect(screen, self.border_color, self.contents, self.border_height)
        screen.blit(self.border_text, (10, ROOM_HEIGHT + 1))
        screen.blit(self.status, (10, SCREEN_HEIGHT - self.border_height + 1))
        screen.blit(self.score, (200, SCREEN_HEIGHT - self.border_height + 1))
        x = 22
        y = SCREEN_HEIGHT - self.border_height - 16
        for item in self.items:
            if game.cur_item != item:
                x = items[item].inventory.update_inventory(screen, x, y)
        
    def update_status(self):
        self.status = self.font.render("You are in the " + game.cur_room.title, True, pygame.Color("black"), self.border_color)
        self.dirty = True

    def update_score(self, points):
        game.score += points
        self.score = self.font.render("Score: " + str(game.score), True, pygame.Color("black"), self.border_color)
        self.dirty = True

    def add(self, item):
        self.items.append(item)
        self.dirty = True

    def remove(self, item):
        self.items.remove(item)
        self.dirty = True

    def mouse_click(self, pos):
        if game.is_drag:   #just in case
            game.is_drag = False
            return
        x = 20
        y = SCREEN_HEIGHT - self.border_height - 16
        for item in self.items:
            #print items[item].inventory.rect
            if items[item].inventory.disp_rect.collidepoint(pos):
                game.is_drag = True
                game.cur_item = item
                items[game.cur_item].inventory.rect[0] = pos[0]
                items[game.cur_item].inventory.rect[1] = pos[1]
                allsprites.add(items[game.cur_item].inventory, layer = 2)
                break

    def mouse_motion(self, pos):
        items[game.cur_item].inventory.rect[0] = pos[0]
        items[game.cur_item].inventory.rect[1] = pos[1]
        self.dirty = True

    def mouse_release(self, pos):    
        game.is_drag = False
        allsprites.remove(items[game.cur_item].inventory)
        if pos[1] <= ROOM_HEIGHT:
            game.cur_room.use(game.cur_item)
        game.cur_item = ""
        self.dirty = True

class InvItem(pygame.sprite.DirtySprite):
    def __init__(self, name, image):
        pygame.sprite.DirtySprite.__init__(self)
        self.name = name 
        filename = "objects/" + image + ".png"
        self.image = load_image(filename, -2)
        self.rect = self.image.get_rect()
        self.disp_rect = self.rect
        self.font = game.font
        self.title = self.font.render(self.name.title(), True, pygame.Color("black"), pygame.Color("white"))
        #use self.title.get_width() to center the title relative to the icon!
       
    def update(self):
        self.dirty = 1

    def update_inventory(self, screen, x, y):
        self.disp_rect[0] = x
        self.disp_rect[1] = y - self.rect[3] - 5
        screen.blit(self.image, (x, y - self.rect[3] - 5))
        centerx = (x + self.rect[2])/2
        textpos = self.title.get_rect(centerx=self.rect[2]/2)
        textpos[0] += x
        textpos[1] = y
        screen.blit(self.title, textpos)
        return x + self.rect[2] + 20

class Item:
    def __init__(self, name, room, description=None, messagepic=None, sound=None):
        self.name = name
        self.rect = Rect
        self.description = description
        if messagepic:
            self.messagepic = messagepic
        else:
            self.messagepic = "information"
        self.sound = sound    
        self.pickable = False
        if room != '':
            rooms[room].items.append(name)
    
    def collide(self, item):
        return self.rect.colliderect(items[item].inventory.rect)
        
    def use(self, item):
        #print "Item: Use item " + item + " on " + self.name
        messagebox.display("information", " It is unlikely that using the " + items[item].name.title() + " \n here would do any good. ", "nnogoodthere")

    def mouse_click(self, pos):
        #print "Item: name=" + self.name + " mouseclick!"
        if not self.pickable and self.description:
            messagebox.display(self.messagepic, self.description, self.sound)
            
        #if the item is pickable, fall through to the avatar callback so it moves to the item and picks it
        return not self.pickable

    def move(self, x, y):
        self.rect[0] = x
        self.rect[1] = y


class ItemSprite(pygame.sprite.DirtySprite, Item):
    def __init__(self, name, image, room, position, description=None, messagepic=None, sound=None):
        pygame.sprite.DirtySprite.__init__(self)
        Item.__init__(self, name, room, description, messagepic, sound)
        filename = "objects/" + image + ".png"
        self.image = load_image(filename, -2)
        self.rect = self.image.get_rect()
        self.position = position
        self.rect[0] = self.position[0]
        self.rect[1] = self.position[1]
        self.pickable = False
        self.scenery = False
        
    def update(self):
        self.dirty = 1

class ItemPickable(ItemSprite):
    def __init__(self, name, image, icon, room, position, description=None, messagepic=None, sound=None):
        ItemSprite.__init__(self, name, image, room, position, description, messagepic, sound)
        #self.name = name
        filename = "objects/" + image + ".png"
        self.image = load_image(filename, -2)
        self.rect = self.image.get_rect()
        self.position = position
        self.rect[0] = self.position[0]
        self.rect[1] = self.position[1]
        self.pickable = True
        self.scenery = False
        self.inventory = InvItem(name, icon)

    def pick(self):
        if not self.name in game.cur_room.items:
            return
        if self.description:
            messagebox.display(self.messagepic, self.description, self.sound)
        inventory.add(self.name)
        game.cur_room.items.remove(self.name)
        allsprites.remove(items[self.name])
        if self.name in treasures:
            inventory.update_score(5)
        avatar.moving = False    

    def mouse_click(self, pos):
        game.pending = self.name
        center = [self.position[0] + self.rect[2]/2, self.position[1] + self.rect[3]/2]
        avatar.mouse_click(center)
        return True

    def action(self):
        self.pick()


class ItemScenery(Item):
    def __init__(self, name, room, left, top, right, bottom, description=None, messagepic=None, sound=None):
        Item.__init__(self, name, room, description, messagepic, sound)
        self.scenery = True
        self.rect = Rect(left, top, right-left, bottom-top)

#    def draw(self, screen):
#       pygame.draw.rect(screen, pygame.Color("white"), self.rect, 1)


class Vault(ItemScenery):
    def __init__(self, name, room, left, top, right, bottom, description=None, messagepic=None, sound=None):
        ItemScenery.__init__(self, name, room, left, top, right, bottom, description, messagepic, sound)

    def use(self, item):
        if item in treasures:
            items["banker"].deposit(item)
            return
        items["banker"].use(item)


class FlashingSign(pygame.sprite.DirtySprite, Item):
    def __init__(self, name, image, room, position):
        pygame.sprite.DirtySprite.__init__(self)
        Item.__init__(self, name, room)
        self.images = [load_frame(i, "objects/store_sign00%02d", -2) for i in range(3)]
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.position = position
        self.rect[0] = self.position[0]
        self.rect[1] = self.position[1]
        self.pickable = False
        self.scenery = False
        self.empty = False
        self.counter = 0
 
    def update(self):
        self.dirty = 1
        if self.empty:
            frame = 2
        else:
            frame = (self.counter//40 + 1) % 2
            self.counter += 1
        self.image = self.images[frame]


class Carrots(ItemSprite):
    def __init__(self, name, image, room, position, description, messagepic=None, sound=None):
        ItemSprite.__init__(self, name, image, room, position, description, messagepic, sound)

    def use(self, item):
        if item == 'rabbit':
            game.cur_room.remove_carrots()
            return
        Item.use(self, item)

class Nellan(ItemSprite):
    def __init__(self, name, image, room, position, description=None, messagepic=None, sound=None):
        ItemSprite.__init__(self, name, image, room, position, description, messagepic, sound)

    def use(self, item):
        if item == 'cold milk':
            messagebox.display(self.messagepic, "Eagerly, Nellan drinks the milk and licks\nthe bowl clean. She is purring very loudly.\n\"Thank you, my friend, for bringing me this\nlovely cold milk. I know it was not easy.\nPlease accept this small token of my\nappreciation.", "necoldmilk")
            inventory.remove("cold milk")
            inventory.add("empty bowl")
            game.cur_room.items.append("vase")
            allsprites.add(items["vase"], layer=2)
            return
        elif item == 'warm milk':
            messagebox.display(self.messagepic, "Nellan slowly laps up the milk and\npolitely says, \"Thank you for the milk. \nPity it's so warm.\"", "newarmmilk")
            inventory.remove("warm milk")
            inventory.add("empty bowl")
            return
        else:
            messagebox.display(self.messagepic, "Nellan smiles at you politely and says,\n\"No thank you, I don't need a " + item.title() + ".\"", "nenothanks")


class BlackDoor(ItemSprite):
    def __init__(self, name, image, room, position):
        ItemSprite.__init__(self, name, image, room, position)

    def mouse_click(self, pos):
        return False   #ignore click on this item so the regular door handling works


class Chair(ItemScenery):
#This item has an action on click but it only activates when the avatar comes close
    def __init__(self, name, room, left, top, right, bottom, center, description=None, messagepic=None, sound=None):
        ItemScenery.__init__(self, name, room, left, top, right, bottom, description, messagepic, sound)
        self.center = center
            
    def mouse_click(self, pos):
        game.pending = self.name
        avatar.mouse_click(self.center)
        return True

    def action(self):
        messagebox.display(self.messagepic, self.description, self.sound)


class Magazine(ItemPickable):
    def __init__(self, name, image, icon, room, position, description=None, messagepic=None, sound=None):
        ItemPickable.__init__(self, name, image, icon, room, position, description, messagepic, sound)

    def mouse_click(self, pos):
        game.pending = self.name
        avatar.mouse_click(self.position)
        return True

    def action(self):
        self.pick()


class BowlMachine(ItemScenery):
    def __init__(self, name, room, left, top, right, bottom, description, messagepic=None, sound=None):
        ItemScenery.__init__(self, name, room, left, top, right, bottom, description, messagepic, sound)
        self.position = (left, top)

    def use(self, item):
        if item == 'coupon':
            game.pending = self.name
            avatar.mouse_click((393, 414))
            return True
        Item.use(self, item)

    def action(self):
        messagebox.display("bowl", "The coupon disappears into the slot,\nand the machine blinks and whirrs for \na few seconds.\nCLUNK!  A shiny new bowl drops into\nthe dispenser.", "nbowlclank")
        inventory.remove("coupon")
        game.cur_room.items.append("empty bowl")
        allsprites.add(items["empty bowl"], layer=1)
        items["flashing sign"].empty = True
        self.description = "This is a bowl vending machine.\nIt does not appear to be working \nright now."
        self.sound = "nvending"

class MilkMachine(ItemScenery):
    def __init__(self, name, room, left, top, right, bottom, description, messagepic=None, sound=None):
        ItemScenery.__init__(self, name, room, left, top, right, bottom, description, messagepic, sound)
        self.position = (left, top)

    def use(self, item):
        if item == 'empty bowl':
            game.pending = self.name
            avatar.mouse_click((341, 413))
            return True
        elif item == 'cold milk' or item == 'warm milk':
            return True
        Item.use(self, item)

    def action(self):
        messagebox.display("milk_pouring", "     GLUG, GLUG, PLINK!\nYour bowl is now filled with \nfresh, cold milk.", "nfullmilk")
        inventory.remove("empty bowl")
        game.cur_room.items.append("cold milk")
        allsprites.add(items["cold milk"], layer=1)


class MagicChair(Chair):
    def __init__(self, name, room, left, top, right, bottom, destination, center, description=None, messagepic=None, sound=None):
        Chair.__init__(self, name, room, left, top, right, bottom, center, description, messagepic, sound)
        self.destination = destination
            
    def action(self):
        game.cur_room.enter(False)  #exit
        game.cur_room = rooms[self.destination];
        inventory.update_status()
        game.cur_room.enter(True)


class Banker(pygame.sprite.DirtySprite, Item):
    def __init__(self, name, image, room, position, description, messagepic=None, sound=None):
        pygame.sprite.DirtySprite.__init__(self)
        Item.__init__(self, name, room, description, messagepic, sound)
        #self.name = name
        self.images = [load_frame(i, "objects/banker00%02d", -2) for i in range(6)]
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.positions = [(210,200), (231,200), (220,200), (220,200), (220,190)]
        self.position = position
        self.rect[0] = self.position[0]
        self.rect[1] = self.position[1]
        self.pickable = False
        self.scenery = False
        self.visits = 0
        self.description = "The man behind the counter looks happy to \nsee you. \"Hello! My name is Mr. Klinkoyn. \nI've lost four treasures somewhere in this \nhouse.  Could you please help me find them?\""
        self.sound = "kwelcome8"
        self.treasure_message = {"egg"  : "The banker, Mr. Klinkoyn, looks happy to\nsee you. \"Ah!  You found my Golden Egg! \nThank you for returning it.\"",
                                 "jewel": "The banker, Mr. Klinkoyn, looks happy to\nsee you. \"I've been missing this Jewel! \nThank you for finding it.\"",
                                 "coin" : "The banker, Mr. Klinkoyn, looks happy to\nsee you. \"This coin has been in the family \nfor a long time.  I'm glad you found it.\"",
                                 "vase" : "The banker, Mr. Klinkoyn, looks happy to\nsee you. \"Thank you for finding this \nvase.  It had been missing a long time!\""}
        
    def update(self):
        self.dirty = 1

    def enter(self):
        frame = self.visits % 5
        self.image = self.images[frame]
        self.position = self.positions[frame]
        self.rect[0], self.rect[1] = self.position[0], self.position[1]
        self.visits += 1

    def standup(self):
        self.image = self.images[5]
        self.position = (214,199)
        self.rect[0], self.rect[1] = self.position[0], self.position[1]

    def use(self, item):
        if item in treasures:
            self.deposit(item)
            return
        messagebox.display(self.messagepic, "Mr. Klinkoyn smiles at you politely \nand says, \"You can keep that.\"", "knothankyou")

    def deposit(self, item):
        self.standup()
        inventory.remove(item)
        allsprites.remove(items[item])
        sound = "k_" + item
        messagebox.display(self.messagepic, self.treasure_message[item], sound)        
        inventory.update_score(20)

    def mouse_click(self, pos):
        messagebox.display(self.messagepic, self.description, self.sound)
        self.description = "Mr. Klinkoyn smiles and says, \n\"I hope you are enjoying your adventure.\""
        self.sound = "neadventure"
        return True
            
        
class Door:
    def __init__(self, perimeter, large_perimeter, dest, target, newpos):
        self.perimeter = perimeter
        self.large_perimeter = large_perimeter
        self.dest = dest
        self.target = target
        self.newpos = newpos
        self.locked = False

class Room:
    def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
        self.name = name
        self.title = name.replace('_', ' ').title()
        filename = "backgrounds/" + image + ".png"
        self.image = load_image(filename, None)
        self.rect = self.image.get_rect()
        self.perimeter = perimeter
        self.large_perimeter = large_perimeter
        self.doors = doors
        self.items = []
        self.visited = False
        self.description = description
        if messagepic:
            self.messagepic = messagepic
        else:
            self.messagepic = "information"
        self.sound = sound
        
    def draw(self, screen):
        screen.blit(self.image, (0, 0))
#        pygame.draw.polygon(screen, pygame.Color("white"), self.perimeter, 2)
#        pygame.draw.polygon(screen, pygame.Color("red"), self.large_perimeter, 2)
#        for obj in self.items:  #for debugging
#            print obj
#            if items[obj].scenery:
#                items[obj].draw(screen)

    def enter(self, mode):
        #mode=True - enter, mode=False - exit
        
        #print "enter room :" + self.name + " mode=" + str(mode)
        #print "items in room: " + str(self.items)

        if mode:
            self.draw(game.screen)
            allsprites.clear(game.screen, self.image)
            allsprites.change_layer(avatar, 2)
            for item in self.items:
                if not items[item].scenery:
                    allsprites.add(items[item], layer = 1)
            if not self.visited:
                self.visited = True
                if self.description:
                    messagebox.display(self.messagepic, self.description, self.sound)
        else:
            for item in self.items:
                if not items[item].scenery:
                    allsprites.remove(items[item])
                    

    def mouse_click(self, pos):
        for door in self.doors:
            if not door.locked and inside_polygon(pos[0], pos[1], door.large_perimeter):
                avatar.goto(door.target)
                return
        for obj in reversed(self.items):   #reversed, so the newest objects have priority (e.g. bowl with milk)
            if items[obj].rect.collidepoint(pos):
                if items[obj].mouse_click(pos):
                    return
        avatar.mouse_click(pos)
     
    def use(self, item):
        #print "Room " + game.cur_room.name + ": use " + items[item].name
        for obj in self.items:
            if items[obj].collide(item):
                return items[obj].use(item)
        if avatar.collide(item):
            return avatar.use(item)
        messagebox.display("information", " It is unlikely that using the " + items[item].name.title() + " \n here would do any good. ", "nnogoodthere")
        
    def update(self):
        pass


class GoldRoom(Room):
     def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
         Room.__init__(self, name, image, perimeter, large_perimeter, doors, description, messagepic, sound)
         self.first = True   #like visited, but only with rabbit in inventory

     def enter(self, mode): 
         if "rabbit" in inventory.items and self.first:
             inventory.remove("rabbit")
             self.items.append("egg")
             self.items.append("rabbit")
             items["rabbit"].move(263, 330)
             messagebox.display("chula_egg", "Chula hops out of your arms and runs \noff.  A moment later she returns with\na large golden egg.", "nchulaegg")
             self.first = False 
         Room.enter(self, mode)             

class Bank(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors)

    def enter(self, mode):
        Room.enter(self,mode)
        if mode:
            items["banker"].enter()


class CatRoom(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors, description, messagepic, sound)

    def enter(self, mode):
        Room.enter(self, mode)
        if mode:
            allsprites.change_layer(avatar, 0)

class MilkRoom(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors)

    def update(self):
        if avatar.pos[1] > 450:
            allsprites.change_layer(avatar, 2)
        else:
            allsprites.change_layer(avatar, 0)

    def remove_carrots(self):
        allsprites.remove(items["carrots"])
        self.items.remove("carrots")
        self.perimeter[2] = (670,447)
        self.perimeter[3] = (644,410)
        inventory.remove("rabbit")
        self.items.append("rabbit")
        allsprites.add(items["rabbit"])
        items["rabbit"].move(130, 413)
        messagebox.display("chula_carrot", "Chula hops out of your arms and\nhungrily devours the huge pile\nof carrots.  She smacks her lips\nand says, \"Thanks, my friend! What\na delicious treat!\"  Then she\nscampers off to a corner to rest.", "eat_carrots")


class Office(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors, description, messagepic, sound)

    def enter(self, mode):
        Room.enter(self,mode)
        if not mode and avatar.pos[1] > 480: #exit through the door
            rooms["white_room"].unlock_door()


class HotRoom(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors, description, messagepic, sound)

    def enter(self, mode):
        Room.enter(self,mode)
        if mode and "cold milk" in inventory.items:
            messagebox.display("milk_warmed", "Because this room is so hot, the\nmilk in your bowl becomes warm.", "nwarmmilk")
            inventory.remove("cold milk")
            inventory.add("warm milk")


class WhiteRoom(Room):
    def __init__(self, name, image, perimeter, large_perimeter, doors, description=None, messagepic=None, sound=None):
        Room.__init__(self, name, image, perimeter, large_perimeter, doors, description, messagepic, sound)
        self.doors[2].locked = True

    def unlock_door(self):
        if self.doors[2].locked:
            self.doors[2].locked = False
            self.items.append("black door")
            self.items.remove("locked door")
        

class Avatar(pygame.sprite.DirtySprite, Item):
    def __init__(self) :
        pygame.sprite.DirtySprite.__init__(self)
        Item.__init__(self, "avatar", "")
        self.pos = [550, 450]
        self.images = [load_frame(i, "tux/tux00%02d", -1) for i in range(16)]
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.size = (94, 120)
        self.moving = False
        self.velocity = [0,0]
        self.target = [-1, -1]
        self.p = [550, 450]
        self.cnt = 0
        self.dir = 0
        self.step = 4
        self.tick = 0
        self.delay = 4
        #variables to choose set of tiles for avatar for each direction
        self.frame_counter = {0:(0,2),  #front-left
                              1:(2,4),  #left
                              2:(6,2),  #front-right
                              3:(8,4),  #right
                              4:(12,4)  #back
                              } 
        #1st val: index for frame_counter; 2: x-velocity, 3: y-velocity
        self.keys_dirs = {K_UP:(4,0,-1),
                     K_DOWN:(0,0,1),
                     K_LEFT:(1,-1,0),
                     K_RIGHT:(3,1,0),
                     K_HOME:(1,-1,-1),
                     K_END:(1,-1,1),
                     K_PAGEUP:(3,1,-1),
                     K_PAGEDOWN:(3,1,1)
                     }

    def new_room(self) :    
        self.p[0] = self.pos[0] + self.velocity[0]
        self.p[1] = self.pos[1] + self.velocity[1]
        for door in game.cur_room.doors:
            if inside_polygon(self.p[0], self.p[1], door.perimeter) and not door.locked:
                game.cur_room.enter(False)  #exit
                game.cur_room = rooms[door.dest];
                inventory.update_status()
                game.cur_room.enter(True)
                self.pos[0] = door.newpos[0]
                self.pos[1] = door.newpos[1]
                self.moving = False
                self.target = [-1, -1]
                return True
        return False    
       
    def update(self):
        self.dirty = 1
        if self.moving and not self.new_room():
            self.p[0] = self.pos[0] + self.velocity[0]
            self.p[1] = self.pos[1] + self.velocity[1]

            if self.target[0] >= 0 and (abs(self.target[0] - self.p[0]) < 3 and abs(self.target[1] - self.p[1]) < 3):
                self.moving = False
            elif not inside_polygon (self.p[0], self.p[1], game.cur_room.perimeter):
                self.moving = False
            else:    
                self.pos[0] += self.velocity[0]
                self.pos[1] += self.velocity[1]
            

        self.rect[0], self.rect[1] = self.pos[0] - self.size[0]/2, self.pos[1] - self.size[1] + 2

        if self.moving:
            #see if the avatar is close to any pickable object (only if moving, to avoid auto-picking objects that appear after actions
            for item in reversed(game.cur_room.items):
                obj = items[item]
                if not obj.pickable:
                    continue
                if obj.rect.colliderect(self.rect):
                    obj.pick()
                    self.moving = False
                    break
            
            self.tick += 1
            if self.tick < self.delay:
                return
            self.tick = 0

            (a, b) = self.frame_counter[self.dir]
            self.cnt = (a + (self.cnt + 1) % b)
            self.image = self.images[self.cnt]
            
            game.cur_room.update()
        else:
            if game.pending != '':
                items[game.pending].action()
                game.pending = ''
                


    def mouse_click(self, pos):
        if self.moving:
            self.moving = False
        else:
            self.goto(pos)

    def goto(self, pos):
        self.target[0], self.target[1] = pos[0], pos[1]
        diffx = self.target[0] - self.pos[0]
        diffy = self.target[1] - self.pos[1]
        dist = math.sqrt(diffx * diffx + diffy * diffy)
        steps = dist / self.step

        self.velocity[0] = diffx / steps
        self.velocity[1] = diffy / steps
        angle = math.atan2(diffy, diffx)
        angle1 = angle % (math.pi * 2)
        #find which direction the avatar is facing
        if angle1 >= math.pi/4 and angle1 < math.pi/2:
            self.dir = 2 #front-right
        elif angle1 >= math.pi/2 and angle1 < 3*math.pi/4:
            self.dir = 0 #front-left
        elif angle1 >= 3*math.pi/4 and angle1 < 5*math.pi/4:
            self.dir = 1 #left
        elif angle1 >= 5*math.pi/4 and angle1 <= 7*math.pi/4:
            self.dir = 4 #up
        else:
            self.dir = 3 #right   (angle1 > 7*pi/4 or angle1 < pi/4)
        self.cnt = self.frame_counter[self.dir][0]
        self.image = self.images[self.cnt]
        self.moving = True

    def key_callback(self, key):
        if not key in self.keys_dirs:
            return
        if self.moving and self.dir == self.keys_dirs[key][0]:
            self.moving = False
            return
        self.dir = self.keys_dirs[key][0]
        self.cnt = self.frame_counter[self.dir][0]
        self.image = self.images[self.cnt]
        self.velocity[0] = self.keys_dirs[key][1] * self.step
        self.velocity[1] = self.keys_dirs[key][2] * self.step
        self.moving = True

    def use(self, item):
        #print "Item: Use item " + item + " on avatar"
        if item == 'cold milk' or item == 'warm milk':
            messagebox.display("information", "Sluuuuurrp! You drink all the milk \nfrom your bowl. Good for you! \nLooks like you're getting your minimum \ndaily requirement of calcium today! ", "ncalcium")
            inventory.remove(item)
            inventory.add("empty bowl")
            return True
        Item.use(self,item)
        

def create_rooms():
    global rooms
    
    cat_room_doors = []
    cat_room_doors.append(Door([(316,360), (264,182), (286,360)], [(356,360), (351,182), (249,182), (257,360)], "hot_room", (304, 351), (218, 490)))
    cat_room_doors.append(Door([(197,395), (164,395), (135,443)], [(168,395), (156,174), (88,164), (106,443)], "green_room", (119, 423), (635, 430)))
    cat_room = CatRoom("cat_room", "cat_room", \
                    [(45,510), (291,510), (300,467), (530,360), (316,360), (264,337), (286,360), (244,360), (197,395), (164,395), (135,443)],\
                    [(16,510), (331,510), (340,467), (570,360), (356,360), (351,182), (249,182), (257,360), (215,360), (168,395), (156,174), (88,164), (106,443)],\
                    cat_room_doors,
                    description="Wow!  You've never seen a cat this big\nbefore.  She smiles at you and says, \"Hello,\nmy friend.  My name is Nellan.  I'm so thirsty.\nWould you be kind enough to bring me some\ncold milk? I must warn you; it won't be easy.\nBut I think you are clever enough to do it.\"",
                    messagepic="nellan_hello", sound="ncat")
    
    store_doors = []
    store_doors.append(Door([(195,401), (167,401), (146,439)], [(166,401), (158,195), (106,188), (117,439)], "milk_room", (138, 401), (658, 440)))
    store = Room("store", "store", \
                 [(46,510), (745,510), (625,424), (567,424), (346,399), (346,382), (236,382), (195,401), (167,401), (146,439)],\
                 [(179,510), (785,510), (665,424), (607,424), (386,399), (386,382), (207,382), (166,401), (158,195), (106,188), (117,439)],\
                 store_doors)
    
    milk_room_doors = []
    milk_room_doors.append(Door([(394,510), (410,490), (474,510)], [(358,510), (358,490), (474,490), (474,510)], "gold_room", (418, 504), (405, 354)))
    milk_room_doors.append(Door([(670,447), (644,410), (615,408)], [(710,447), (717,201), (660,205), (655,408)], "store", (686, 419), (170, 422)))
    milk_room = MilkRoom("milk_room", "milk_room", \
                     [(46,510), (752,510), (670,455), (480,455), (615,408), (560,370), (344,370), (344,410), (234,435), (167,420)],\
                     [(179,510), (792,510), (710,455), (717,201), (660,205), (655,408), (600,370), (315,370), (315,410), (205,435), (138,420)],\
                     milk_room_doors)
    
    hot_room_doors = []
    hot_room_doors.append(Door([(176,413), (150,413), (130,447)], [(147,413), (137,191), (88,184), (101,447)], "gold_room", (121, 413), (635, 385)))
    hot_room_doors.append(Door([(157,510), (218,493), (279,510)], [(157,510), (157,492), (279,492), (279,510)], "cat_room", (218, 505), (300, 360)))
    hot_room = HotRoom("hot_room", "hot_room", \
                    [(46,510), (749,510), (553,370), (237,370), (176,413), (150,413), (130,447)],\
                    [(17,510), (789,510), (593,370), (208,370), (147,413), (137,191), (88,184), (101,447)],\
                    hot_room_doors,
                    description="Whew! It's hot in here!", messagepic="thermometer", sound="nhotinhere")
    
    gold_room_doors = []
    gold_room_doors.append(Door([(197,358), (155,348), (109,381)], [(168,358), (163,168), (72,154), (80,381)], "white_room", (127, 370), (580, 420)))
    gold_room_doors.append(Door([(421,350), (405,173), (382,350)], [(456,338), (454,173), (351,172), (353,337)], "milk_room", (401, 335), (417, 485)))
    gold_room_doors.append(Door([(697,384), (657,357), (609,361)], [(737,384), (741,152), (651,166), (649,361)], "hot_room", (709, 371), (170, 428)))
    gold_room = GoldRoom("gold_room", "gold_room", \
                     [(78,510), (400,530), (722,510), (724,395), (697,384), (657,357), (609,361), (510,345), (421,338), (405,173), (382,337), (291,342), (197,358), (155,348), (109,381), (75,395)],\
                     [(49,510), (400,530), (762,510), (764,395), (737,384), (740,153), (651,166), (649,361), (550,345), (456,338), (454,173), (351,172), (353,337), (262,342), (168,358), (164,167), (73,154), (80,381), (46,395)],\
                     gold_room_doors)
    
    office_doors = []
    office_doors.append(Door([(343,510), (400,490), (454,510)], [(343,510), (343,492), (454,492), (454,510)], "white_room", (396, 505), (400, 370)))
    office = Office("office", "office", \
                  [(44,510), (742,510), (718,496), (602,496), (552,430), (460,438), (470,420), (200,475), (138,430)],\
                  [(15,510), (782,510), (758,496), (642,496), (592,430), (500,438), (441,420), (200,475), (109,430)],\
                  office_doors,
                  description="WOW!  That chair must have been magic!\nYou find yourself in an old, dusty office.\nYou get the impression that no one has set \nfoot inside this room for many years.",
                  messagepic="sitting_in_office", sound="noffice")
    
    bank_doors = []
    bank_doors.append(Door([(539,355), (542,191), (436,191), (435,355)], [(539,355), (542,191), (436,191), (435,355)], "white_room", (488, 345), (480, 485)))
    bank = Bank("bank", "bank", \
                [(379,510), (736,510), (552,355), (465,355), (465,345), (510,345), (510,355), (410,355)],\
                [(347,510), (778,510), (588,355), (539,355), (542,191), (436,191), (435,355), (377,353)],\
                bank_doors)
    
    green_room_doors = []
    green_room_doors.append(Door([(681,460), (650,411), (616,411)], [(721,460), (727,179), (659,190), (656,411)], "cat_room", (690, 428), (180, 420)))
    green_room = Room("green_room", "green_room", \
                      [(50,510), (745,510), (681,460), (650,411), (616,411), (594,400), (211,400)],\
                      [(21,510), (785,510), (721,460), (727,179), (659,190), (656,411), (634,400), (182,400)],\
                      green_room_doors)
    
    white_room_doors = []
    white_room_doors.append(Door([(641,430), (641,416), (598,397)], [(678,427), (687,205), (642,206), (636,395)], "gold_room", (669, 414), (163, 380)))
    white_room_doors.append(Door([(401,510), (401,490), (578,490), (578,510)], [(435,510), (435,490), (549,490), (549,510)], "bank", (490, 505), (480, 360)))
    white_room_doors.append(Door([(476,365), (398,192), (316,365)], [(445,354), (446,192), (352,192), (353,354)], "office", (398, 347), (400, 485)))
    white_room = WhiteRoom("white_room", "white_room", \
                      [(49,510), (751,510), (641,430), (641,416), (598,397), (549,365), (476,365), (398,365), (316,365), (250,365)],\
                      [(20,510), (791,510), (681,430), (687,205), (642,206), (638,397), (589,365), (447,365), (446,192), (352,192), (356,365), (221,365)],\
                      white_room_doors,
                      description="A fluffy white rabbit hops up to you\nand says, \"Hi, my name's Chula!  Welcome \nto the Enchanted House, my friend!\"\nShe gives you a hug and says, \"If you\npick me up and carry me to the room\nto the right, I'll give you a\npresent that I saved for you.\"",
                      messagepic="chula_hello", sound="hiculla")
    
    rooms = {"cat_room" : cat_room,
             "white_room" : white_room,
             "milk_room" : milk_room,
             "hot_room" : hot_room,
             "gold_room" : gold_room,
             "office" : office,
             "bank" : bank,
             "green_room": green_room,
             "store": store}


def create_items():
    global items, treasures

    treasure_message = "This is one of the four treasures\nin the Enchanted House.  You can \nget points for treasures by\ndepositing them in the Bank."
    treasure_sound = "n4treasures"

    items = {"rabbit" : ItemPickable("rabbit", "chula", "chula_icon", "white_room", (168, 416)),
             "egg" : ItemPickable("egg", "egg", "egg_icon", "", (363, 380), description=treasure_message, messagepic="egg", sound=treasure_sound),
             "jewel" : ItemPickable("jewel", "jewel", "jewel_icon", "store", (680, 444), description=treasure_message, messagepic="jewel", sound=treasure_sound), 
             "coin" : ItemPickable("coin", "coin", "coin_icon", "green_room", (200, 423), description=treasure_message, messagepic="coin", sound=treasure_sound),
             "vase" : ItemPickable("vase", "vase", "vase_icon", "", (280, 426),  description=treasure_message, messagepic="vase", sound=treasure_sound),
             "counter_top" : Vault("counter_top", "bank", 54, 82, 371, 503, description="You can get points by returning\ntreasures to Mr. Klinkoyn.", sound="n_points"),
             "vault" : Vault("vault", "bank", 54, 82, 181, 332, description="That is the Bank's vault.\nThere is no way for you to open it.", sound="nvalt"),
             "brigham" : ItemScenery("brigham", "bank", 650, 85, 741, 212, description="A portrait of a dignified-looking\ngentleman hangs on the east wall.", sound="n_brigham"),
             "banker" : Banker("banker", "banker", "bank", (210, 200), description="", messagepic="banker"),
             "milk machine" : MilkMachine("milk machine", "milk_room", 176, 206, 313, 418, description="This is a milk cooler.  If you have\nan empty bowl, you could fill it with\nmilk from here.", sound="nmilkcooler"),
             "bowl machine" : BowlMachine("bowl machine", "store", 386, 135, 603, 385, description="This is a bowl vending machine.\nIf you have a coupon, you could drop\nyour coupon into the machine to\nreceive a new clean bowl.", sound="ncoupond"),
             "carrots" : Carrots("carrots", "carrots", "milk_room", (520, 255), description="Wow!  There is a huge pile of carrots\nblocking the doorway. It's too heavy\nto move, too big to climb over, and\ntoo much for you to eat all by yourself.\nWhat do you think you should do?", sound="ncarrotsblocking"),
             "coupon" : ItemPickable("coupon", "coupon", "coupon_icon", "hot_room", (550,423), description="What's this? A coupon for a free \nbowl!  If you insert this coupon\ninto any vending machine, you \nwill get a nice clean bowl.", sound="ncoupon"),
             "empty bowl": ItemPickable("empty bowl", "bowl", "bowl_icon", "", (461,322)),
             "cold milk": ItemPickable("cold milk", "bowl_cold_milk", "bowl_cold_milk_icon", "", (233,376)),
             "warm milk": ItemPickable("warm milk", "bowl_warm_milk", "bowl_warm_milk_icon", "", (290,460)),
             "magazine" : Magazine("magazine", "magazine", "magazine_icon", "office", (209,278), description="Well, what do you know! \nYou found a genuine copy of the \nJuly/August 1980 issue of \"Recreational\nComputing\" magazine!  It looks\nstrangely familiar to you, although\nit's hard to say why.", messagepic="magazine", sound="nmagazine"),
             "nellan" : Nellan("nellan", "nellan", "cat_room", (300, 30), description="Nellan smiles at you politely and says,\n\"I hope you are enjoying your adventure.\"", messagepic="nellan_hello", sound="neadventure"),
             "black door" : BlackDoor("black door", "black_door", "", (352, 191)),
             "locked door" : ItemScenery("locked door", "white_room", 352, 192, 446, 354, description="Try as you might, the door is solidly \nlocked shut. Perhaps it can only be \nunlocked from the other side...?", sound="nlockeddoor"),
             "left chair" : Chair("left chair", "green_room", 238, 242, 347, 388, (289, 413), description="Ouch!  This chair is too hard!", messagepic="sitting_on_chair_1", sound="nouch"),
             "middle chair" : Chair("middle chair", "green_room", 367,244, 467,389, (412, 413), description="Oomph!  This chair is too soft!", messagepic="sitting_on_chair_2", sound="noomph"),
             "right chair" : MagicChair("right chair", "green_room", 486, 244, 589,390, "office", (487, 446), description="As you settle down onto the chair, you\nstart feeling dizzy.  The room begins\nto spin around you, and suddenly everything \ngoes very dark...", messagepic="sitting_on_chair_3", sound="nmagicchair"),
             "chair" : MagicChair("chair", "office", 450, 251, 561, 431, "green_room", (487, 446), description="You sit in the chair, fully expecting\nto be magically transported back to\nthe Green Room.", messagepic="sitting_in_office", sound="nbacktogreenroom"),
             "cow painting" : ItemScenery("cow painting", "milk_room", 405,147,536,243, description="       \"MOO!\"       ", sound="moo"),
             "trs80": ItemScenery("trs80", "office", 280,217,421,286, description="On the desk sits a very old\nTRS-80 Model 3 computer. You\nhaven't seen one of these relics\nin a very long time.  Sadly, it\ndoes not appear to be working now.", sound="ncomputer"),
             "office window": ItemScenery("office window", "office", 412,97,550,217, description="The dusty window offers the only \nview of the outside world.\nUnfortunately, you cannot make\nout any details.", sound="nwindow"),
             "bookshelf" : ItemScenery("bookshelf", "office", 574,84,790,407, description="The bookshelf contains several thick,\nweighty volumes. None of them look \nvery interesting, though.", sound="nbooks"),
             "flashing sign": FlashingSign("flashing sign", "store_sign0000", "store", (463,259)) 
                                       
             }

    treasures = ("egg", "jewel", "coin", "vase")


    
def main():
    global inventory, avatar, allsprites, messagebox, game

    pygame.init()
    game = Game()
    game.font = pygame.font.Font("ttf-bitstream-vera-1.10/Vera.ttf", 12)
    game.screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
    pygame.display.set_caption('Thirsty Nellan')
    pygame.mouse.set_visible(1)
    clock = pygame.time.Clock()
    if not pygame.mixer:
        print "Warning: can't find sound mixer!"
    else:
        try:
            pygame.mixer.init()
            if pygame.mixer.get_init():
                game.sound_on = True
            else:
                print "Warning: can't initialize sound mixer!"
        except pygame.error:
            print "Warning: can't initialize sound mixer!"

    background = pygame.Surface(game.screen.get_size())
    background = background.convert()
    game.screen.blit(background, (0, 0))
    pygame.display.flip()

    create_rooms()
    create_items()
    game.cur_room = rooms["bank"]
    inventory = Inventory()
    messagebox = MessageBox()
    avatar = Avatar()
    avatar._layered = 2
    allsprites = pygame.sprite.LayeredDirty((avatar))
    game.cur_room.draw(game.screen)
    allsprites.clear(game.screen, game.cur_room.image)
    game.cur_room.enter(True)
    inventory.draw(game.screen)

    splash = Splash()
    pygame.time.set_timer(pygame.USEREVENT + 1, 5000)
    going = True
    splash.draw(game.screen)
    pygame.display.flip()
    while(going):
        clock.tick(25)
        for event in pygame.event.get():
            if event.type == pygame.USEREVENT + 1 or event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
                going = False
    pygame.time.set_timer(pygame.USEREVENT + 1, 0)

    going = True
    while going:
        clock.tick(25)

        if messagebox.active:
            messagebox.draw(game.screen)
            for event in pygame.event.get():
                if event.type == QUIT:
                    going = False
                elif event.type == KEYDOWN:
                   if event.key == K_RETURN:
                       messagebox.hide()
                elif event.type == MOUSEBUTTONDOWN:
                    messagebox.mouse_click(event.pos)
        else:        
            for event in pygame.event.get():
                if event.type == QUIT:
                    going = False
                elif event.type == KEYDOWN:
                    avatar.key_callback(event.key)
                elif event.type == MOUSEBUTTONDOWN:
                    if event.pos[1] > ROOM_HEIGHT:
                        inventory.mouse_click(event.pos)
                    else:    
                        game.cur_room.mouse_click(event.pos)
                elif event.type == MOUSEBUTTONUP:
                    if game.is_drag:
                        inventory.mouse_release(event.pos)
                
                elif event.type == MOUSEMOTION:
                    if game.is_drag:
                        inventory.mouse_motion(event.pos)

            allsprites.update()
            inventory.draw(game.screen)
            rects = allsprites.draw(game.screen)
            pygame.display.update(rects)
                                    
        pygame.display.flip()

    pygame.quit()

if __name__ == '__main__':
    main()


